{% extends "core/site-base.html" %}

{% block tabtitle %}
Simmate
{% endblock %}

{% block banner %}
{% include "core/header.html"%}
{% endblock %}


{% block body %}

<!-- NOTE: for this section to work, be sure to read datatable.html -->

<div class="container p-4 bg-light border" id="search-results">

  <div class="container p-4">

    <!-- Breadcrumb links -->
    <nav aria-label="breadcrumb">
      <ol class="breadcrumb">
        <li class="breadcrumb-item"><a href="{% url 'local_calculations' %}">Workflows</a></li>
        <li class="breadcrumb-item"><a href="{% url 'relaxations' %}">Relaxations</a></li>
        <li class="breadcrumb-item"><a href="{% url 'mit_about' %}">MIT-Project Quality</a></li>
        <li class="breadcrumb-item"><a href="{% url 'mit_all' %}">Results</a></li>
        <li class="breadcrumb-item active">mitrelax-{{ calculation.id }}</li>
      </ol>
    </nav>

    {{ figure_convergence_html | safe }}

  </div>

</div>


{% endblock %}

{% block extrafooter %}
{% include "core/datatable.html" %}
{% endblock %}