{% extends "core/site-base.html" %}

{% block tabtitle %}
Simmate
{% endblock %}

{% block banner %}
{% include "core/header.html"%}
{% endblock %}


{% block body %}

<div class="container p-4 bg-light border" id="search-results">

  <!-- Breadcrumb links -->
  <nav aria-label="breadcrumb">
    <ol class="breadcrumb">
      <li class="breadcrumb-item"><a href="{% url 'local_calculations' %}">Workflows</a></li>
      <li class="breadcrumb-item"><a href="{% url 'relaxations' %}">Relaxations</a></li>
      <li class="breadcrumb-item active">MIT-Project Quality</li>
    </ol>
  </nav>


  <div class="container p-4">

    <p>
      <b>This page hasn't been created yet</b>
      <i class="bi bi-emoji-frown fs-4"></i>
      </br>
      Once I have time, this will be where you get an overview of this calculation
      and it's results. I'll show everything from the settings used to number of
      calculations ran. For now, just jump to the Submit/Monitor/Results pages instead.
    </p>

    <a role="button" class="btn btn-primary btn" href="{% url 'mit_submit' %}">Submit</a>
    <a role="button" class="btn btn-primary btn" href="https://cloud.prefect.io/simmate/flow/{{ mit_flow_id }}"
      target="_blank">Monitor</a>
    <a role="button" class="btn btn-primary btn" href="{% url 'mit_all' %}">Results</a>

  </div>

  {% endblock %}